// Squawk messenger. 
// Copyright (C) 2019  Yury Gubich <blue@macaw.me>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef CORE_TABLE_H
#define CORE_TABLE_H

#include "database.h"
#include "serializer.h"

class DataBase::_Table {
    friend class DataBase;
protected:
    _Table(const std::string& name, DataBase* parent);
    virtual ~_Table();

    virtual int createTable(MDB_txn * transaction) = 0;
    virtual int drop(MDB_txn * transaction);

public:
    virtual void drop();
    virtual uint32_t count() const;

protected:
    MDB_dbi dbi;
    DataBase* db;
    const std::string name;

protected:
    template <class T>
    int makeTable(MDB_txn* transaction);

    template <class T>
    static std::string toString(const T& value);
};

template <class K, class V>
class DataBase::Table : public DataBase::_Table {
    friend class DataBase;
protected:
    Table(const std::string& name, DataBase* parent);
    ~Table() override;

public:
    virtual void addRecord(const K& key, const V& value);
    virtual bool forceRecord(const K& key, const V& value);     //returns true if there was addition, false if change
    virtual void changeRecord(const K& key, const V& value);
    virtual void removeRecord(const K& key);
    virtual bool checkRecord(const K& key) const;               //checks if there is a record with given key
    virtual V getRecord(const K& key) const;
    virtual std::map<K, V> readAll() const;

protected:
    Serializer<K>* keySerializer;
    Serializer<V>* valueSerializer;

    int createTable(MDB_txn* transaction) override;
};

#include "table.hpp"

#endif // CORE_TABLE_H
